//**********************************************************************
// 	Project: PDM2PCM Project
//	File:  top.v
// 	Description: board test top
//	Author: Noa
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	| Noa		| xx/xx/2019	| Initial version
// v1.0.1	| Noa		| xx/xx/2019	| 
//**********************************************************************
module Top#(
	parameter DATA_WIDTH = 16)(
    input SYM_CLK,
    input SW1,
    input KEY1,
    output out_pcm_sync/* synthesis syn_keep=1 */,
    output out_pcm_valid/* synthesis syn_keep=1 */,
    output signed [DATA_WIDTH-1:0] out_pcm_data/* synthesis syn_keep=1 */
);

//**********************************************************************
// --- Parameter
//**********************************************************************
	localparam MEM_DEPTH = 2048;   
	localparam NUM_CHN = 4;

	
//**********************************************************************
// --- Inter Signal Declaration
//**********************************************************************
	wire clk, ce, rstn;

    wire pdm_clk;
	reg pdm_sclk_r;
    wire pdm_sclk_posedge;

	wire [NUM_CHN-1:0] pdm_data; 
	
    reg [$clog2(MEM_DEPTH):0] drom_addr;
	wire drom_out;
	
//**********************************************************************	
// --- Initial Block
//**********************************************************************
// --- clock divider (50M=>1M)
    clk_div_even #(
		.N(25)
	)u_clk_div_PDM(
		.clk(clk), 
		.rstn(rstn),
		.ce(ce),
		.clk_out(pdm_sclk)
	);

// --- PDM2PCM
	PDM2PCM_Top u_PDM2PCM_Top(
		.clk(clk), 
		.rstn(rstn),
		.ce(ce),
	
		.in_pdm_sclk(pdm_sclk),
		.in_pdm_data(pdm_data),
	
		.out_pcm_valid(out_pcm_valid),
		.out_pcm_sync(out_pcm_sync),
		.out_pcm_data(out_pcm_data)
	);
    assign clk = SYM_CLK;
    assign rstn = KEY1;
    assign ce = SW1;


//**********************************************************************
//----------read data from mem-----------
//**********************************************************************
// --- sclk negedge
    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin
            pdm_sclk_r <= 0;
        end
        else if(ce) begin
            pdm_sclk_r <= pdm_sclk;
        end
    end
    assign pdm_sclk_posedge = pdm_sclk & ~pdm_sclk_r;

// --- rom addr 
    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin
            drom_addr <= MEM_DEPTH - 1;
        end
        else if(ce & pdm_sclk_posedge) begin
            if(drom_addr == MEM_DEPTH - 1) begin
                drom_addr <= 0;
            end
            else begin
                drom_addr <= drom_addr+1;
            end
        end
    end
// --- ip PDM2PCM
    assign pdm_data = {(NUM_CHN){drom_out}};


	
// --- Data_Rom
	Data_pROM data_prom(
        .dout(drom_out), //output [0:0] dout
        .clk(clk), //input clk
        .oce(ce), //input oce
        .ce(ce), //input ce
        .reset(~rstn), //input reset
        .ad(drom_addr) //input [13:0] ad
    );

	



endmodule